<?php

namespace Drupal\element451_forms\Form\Webform;


use Drupal\Core\Form\FormStateInterface;
use Drupal\webform\EntitySettings\WebformEntitySettingsBaseForm;

/**
 * Form class to define custom element settings for Webform.
 */
class WebformEntitySettingsElementForm extends WebformEntitySettingsBaseForm {

  private $elements = [];

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'webform_element_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    /** @var \Drupal\webform\WebformInterface $webform */
    $webform = $this->entity;

    // Element Form ID. Needs to be grabbed from Element
    $form['element451_form_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Element451 Form ID'),
      '#description' => $this->t('Enter the Element451 Form ID for this webform. This can be found in the element451 form URL.'),
      '#placeholder' => $this->t('client.forms.xxxx'),
      '#default_value' => $webform->getThirdPartySetting('element', 'element451_form_id', ''),
    ];

    //Element Fields ID. A must have value for an Element Post.
    $form['element451_fields_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Element451 Fields ID'),
      '#description' => $this->t('This field will be populated after the Generate Fields button has been clicked'),
      '#default_value' => $webform->getThirdPartySetting('element', 'element451_fields_id', ''),
      '#disabled' => TRUE,
    ];

    // Add a button to dynamically create fields based on the Element451 Form ID.
    if($webform->getThirdPartySetting('element', 'element451_form_id', '') !== ''){
      $form['create_fields'] = [
        '#type' => 'submit',
        '#value' => $this->t('Generate/Re-Sync Fields'),
        '#submit' => ['::createFields'], // Call a custom submit handler for this button.
        '#button_type' => 'primary',
      ];
    }

    return parent::form($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $webform = $this->getEntity();
    $webform->setThirdPartySetting('element', 'element451_form_id', $form_state->getValue('element451_form_id'));
    $webform->save();
    $this->messenger()->addStatus($this->t('The Element451 Form ID has been saved.'));
    parent::save($form, $form_state);
  }

  /**
   * Custom submit handler to dynamically create fields.
   */
  public function createFields(array &$form, FormStateInterface $form_state) {

    $webform = $this->getEntity();
    $handlers = $webform->getHandlers();
    $element451_form_id = $webform->getThirdPartySetting('element', 'element451_form_id', '');
    $element451_api = \Drupal::service('element451_forms.element_api');
    $fields = $element451_api->getFormFields($element451_form_id);

    $webform_elements = $webform->getElementsDecoded();
    $element_elements = $this->loopFields($fields);

    $elements = array_merge($webform_elements, $element_elements);

    //Set a Status Field
    $elements['status'] = [
      '#title' => 'Status',
      '#type' => 'hidden',
      '#default_value' => 'pending',
    ];

    //Set Element Connector handler if it already isn't set.
    if(!($handlers->has('element_connector'))){
      $handler_manager = \Drupal::service('plugin.manager.webform.handler');
      // Create webform handler.
      $handler_configuration = [
        'id' => 'element_connector',
        'label' => 'Element Connector',
        'handler_id' => 'element_connector',
        'status' => 1,
        'weight' => 0,
        'settings' => [],
      ];
      $handler = $handler_manager->createInstance('element_connector', $handler_configuration);
      $webform->setOriginalId($webform->id());
      $webform->addWebformHandler($handler);
    }

    $webform->setElements($elements);
    $webform->setThirdPartySetting('element', 'element451_fields_id', $fields['element_fields_id']);
    $webform->save();
  }

  protected function createField($field,  $type, $sub_field = FALSE, $parent_name = '', $parent_slug = ''){
    $machine_name = str_replace("-", "_", $field['name']);
    $this->elements[$machine_name] = [
      '#type' => $type,
      '#title' => $field['label'],
      '#required' => !empty($field['validations'][0]['type']) ?? TRUE,
      '#default_value' => !empty($field['default']) ?? $field['default'],
      '#access' => !empty($field['hidden']) !== TRUE ?? FALSE,
      '#custom_values' => [
        'slug' => !empty($field['slug']) ? $field['slug'] : NULL,
        'name' => $field['name'],
      ],
    ];
    //Add data validation to text fields and text areas
    if($type === 'textfield' || $type === 'textarea'){
        $this->elements[$machine_name]['#pattern'] = "^[a-zA-Z0-9\s.,-]+$";
        $this->elements[$machine_name]['#pattern_error'] = "Can contain only letters, numbers, spaces and interpunction (allowed special characters are the period, comma, and dash)";
    }
    //Check for fields that require options
    if($type === 'checkboxes' || $type === 'radios' || $type === 'select'){
      $options = [];
      if(!empty($field['data_source_settings'])){
        foreach($field['data_source_settings'] as $option){
          $value = $option['value'];
          $text = $option['text'];
          $options[$value] = $text;
        }
      }
      if(!empty($field['options'])){
        foreach($field['options'] as $option){
          $value = $option['value'];
          $text = $option['text'];
          $options[$value] = $text;
        }
      }
      $this->elements[$machine_name]['#options'] = $options;
      if($machine_name === 'user_phone_country_code' || $machine_name === 'primary_phone_country_code'){
        $this->elements[$machine_name]['#options'] = 'country_codes';
      }
    }
    //Check for subfields
    if($sub_field && $parent_name !== '' && $parent_slug !== ''){
      $this->elements[$machine_name]['#custom_values']['parent_name'] = $parent_name;
      $this->elements[$machine_name]['#custom_values']['parent_slug'] = $parent_slug;
    }

  }

  private function loopFields($fields){
    //create webform fields
    foreach ($fields['element_fields'] as $field){
      if(empty($field['subfields'])){
        switch($field['type']){
          case 'text':
            $this->createField($field, 'textfield');
            break;
          case 'textarea':
            $this->createField($field, 'textarea');
            break;
          case 'email':
            $this->createField($field, 'email');
            break;
          case 'datepicker':
            $this->createField($field, 'date');
            break;
          case 'datetime':
            $this->createField($field, 'datetime');
            break;
          case 'select':
            $this->createField($field, 'select');
            break;
          case 'radio':
            $this->createField($field, 'radios');
            break;
          case 'checkbox':
            $this->createField($field, 'checkboxes');
            break;
          case 'boolean':
            $this->createField($field, 'checkbox');
            break;
        }
      }
      else{
        $parent_name = $field['name'];
        $parent_slug = $field['slug'];
        foreach($field['subfields'] as $sub_field){
          switch($sub_field['type']){
            case 'text':
              $this->createField($sub_field, 'textfield', TRUE, $parent_name, $parent_slug);
              break;
            case 'textarea':
              $this->createField($sub_field, 'textarea', TRUE, $parent_name, $parent_slug);
              break;
            case 'email':
              $this->createField($sub_field, 'email', TRUE, $parent_name, $parent_slug);
              break;
            case 'datepicker':
              $this->createField($sub_field, 'date', TRUE, $parent_name, $parent_slug);
              break;
            case 'datetime':
              $this->createField($sub_field, 'datetime', TRUE, $parent_name, $parent_slug);
              break;
            case 'select':
              $this->createField($sub_field, 'select', TRUE, $parent_name, $parent_slug);
              break;
            case 'radio':
              $this->createField($sub_field, 'radios', TRUE, $parent_name, $parent_slug);
              break;
            case 'checkbox':
              $this->createField($sub_field, 'checkboxes', TRUE, $parent_name, $parent_slug);
              break;
            case 'boolean':
              $this->createField($sub_field, 'checkbox', TRUE, $parent_name, $parent_slug);
              break;
          }
        }
      }
    }
    return $this->elements;
  }



}
