<?php

namespace Drupal\element451_forms\Service;

use GuzzleHttp\ClientInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Psr\Log\LoggerInterface;

/**
 * Service class for interacting with the Element451 API.
 */
class ElementApi {

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Config factory interface.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Custom Fields for API
   */

  /**
   * Constructs the Element451Api service.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The Guzzle HTTP client.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */



  public function __construct(ClientInterface $http_client, ConfigFactoryInterface $config_factory, LoggerInterface $logger) {
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
    $this->logger = $logger;
  }

  /**
   * Makes the API call to fetch form fields from Element451.
   *
   * @param string $form_id
   *   The Element451 form ID.
   *
   * @return array
   *   An array of form fields.
   */
  public function getFormFields(string $form_id) {
    $request_url = $this->_create_base_request_URL($form_id);
    try {

      $response = $this->httpClient->get($request_url);

      if ($response->getStatusCode() == 200) {
        $data = json_decode($response->getBody(), TRUE);
        $element_fields_id = $data['data']['main_form']['_field'];
        $fields = $data['data']['main_form']['_field_data']['fields'];
        return ['element_fields_id' => $element_fields_id, 'element_fields' => $fields] ?? [];
      }

    } catch (\Exception $e) {
      $this->logger->error('Element451 API call failed: @message', ['@message' => $e->getMessage()]);
    }

    return [];

  }

  public function postElementSubmission(string $form_id, $data) {

    $request_url = $this->_create_base_request_URL($form_id);
    $config = \Drupal::config('element451_forms.api_settings');
    $bearer_token_config = $config->get('bearer_token');
    $bearer_token = \Drupal::service('key.repository')->getKey($bearer_token_config)->getKeyValue();

    // Prepare the POST request data.
    $post_data = [
      'headers' => [
        'Authorization' => 'Bearer ' . $bearer_token,
        'Accept' => 'application/json',
        'Content-Type' => 'application/json',
      ],
      'json' => [
        'item' => $data,
      ],
    ];

    try {
      // Send the POST request to Element451.
      $response = $this->httpClient->post($request_url, $post_data);

      // Check if the response status code is 200 (success).
      if ($response->getStatusCode() == 200) {
        $response_body = json_decode($response->getBody(), TRUE);

        // Handle the success response.
        \Drupal::messenger()->addStatus('Your submission was successfully submitted.');

        return $response_body;
      }

    }
    catch (\Exception $e) {
      $this->logger->error('Element451 API call failed: @message', ['@message' => $e->getMessage()]);
    }
  }

  private function _create_base_request_URL($form_id){
    $config = \Drupal::config('element451_forms.api_settings');
    $base_url = $config->get('base_url') . '/v2/forms/public';
    $analytics_key_config = $config->get('analytics_key');
    $feature_key_config = $config->get('feature_key');
    $analytics_key = \Drupal::service('key.repository')->getKey($analytics_key_config)->getKeyValue();
    $feature_key = \Drupal::service('key.repository')->getKey($feature_key_config)->getKeyValue();

    $element_url = $base_url . '/' . $form_id
      . '?analytics=' . $analytics_key
      . '&feature=' . $feature_key;

    return $element_url;
  }

}
